package model;

import org.json.JSONArray;
import org.json.JSONObject;

import java.util.HashMap;
import java.util.Map;

public class GameState
{
	long[] fish; //Array of three bitboards: one for every fish value
	boolean can_play_red, can_play_blue; //If the red(blue) player can play or if it has no moves left
	Player current_player;
	Map<Player, int[]> penguins; //For every player, the list of his penguins
	Map<Player, Integer> score; //Score of every player

	public GameState()
	{
		fish = new long[3];
		penguins = new HashMap<>();
		penguins.put(Player.Red, new int[4]);
		penguins.put(Player.Blue, new int[4]);
		score = new HashMap<>();
		score.put(Player.Red, 0);
		score.put(Player.Blue, 0);
	}

	public void update(String jsonString)
	{
		//System.out.println(jsonString);
		JSONObject json = new JSONObject(jsonString);
		can_play_red = json.getJSONObject("can_play").getBoolean("red");
		can_play_blue = json.getJSONObject("can_play").getBoolean("blue");
		current_player = Player.valueOf(json.getString("current_player"));
		fish[0] = json.getJSONObject("bitboards").getLong("onefish");
		fish[1] = json.getJSONObject("bitboards").getLong("twofish");
		fish[2] = json.getJSONObject("bitboards").getLong("threefish");
		score.put(Player.Red, json.getJSONObject("score").getInt("red"));
		score.put(Player.Blue, json.getJSONObject("score").getInt("blue"));
		JSONArray jsonPeng = json.getJSONObject("penguins").getJSONArray("red");
		for (int i = 0; i < jsonPeng.length(); i++)
			penguins.get(Player.Red)[i] = jsonPeng.getInt(i);
		jsonPeng = json.getJSONObject("penguins").getJSONArray("blue");
		for (int i = 0; i < jsonPeng.length(); i++)
			penguins.get(Player.Blue)[i] = jsonPeng.getInt(i);
	}

	;

	/**
	 * @param tileNb The number of the tile we want to know the value of
	 * @return The number of fish on the tile, or 0 if empty
	 */
	public int getNbFish(int tileNb)
	{
		if (((fish[0] >>> tileNb) & 1) == 1)
			return 1;
		else if (((fish[1] >>> tileNb) & 1) == 1)
			return 2;
		else if (((fish[2] >>> tileNb) & 1) == 1)
			return 3;
		return 0;
	}

	/**
	 * @param player The player for which we are searching the penguin position
	 * @param i      The number of the penguin for this player
	 * @return The position (0-59) of the penguin on the board
	 */
	public int getPenguinPos(Player player, int i)
	{ return (penguins.get(player)[i] & 63); }

	/**
	 * @param player The penguin's owner
	 * @param i The number of the penguin for this player
	 * @return the total moves that this penguin can do
	 */
	public int getPenguinTotalMoves(Player player, int i)
	{ return ((penguins.get(player)[i] >>> 6) & 63); }

	public enum Direction
	{
		A,B,C,D,E,F;
	}

	/**
	 * Get maximum number of tiles that a penguin can do in one specific direction
	 */
	public int getNbMoves(Player player, int penguinNumber, Direction direction)
	{
		int shift = 0;
		switch (direction)
		{
			case A:
				shift = 12;
				break;
			case B:
				shift = 15;
				break;
			case C:
				shift = 18;
				break;
			case D:
				shift = 21;
				break;
			case E:
				shift = 24;
				break;
			case F:
				shift = 27;
				break;
		}
		return ((penguins.get(player)[penguinNumber] >>> shift) & 7);
	}

	/**
	 * "I want to move penguin 1 in direction B for 5 steps"
	 * "This is you move number 10 out of all your possible moves"
	 * Note: moves start at 0. Steps start at 1 for this function
	 */
	public int getPlayerMoveNumber(Player player, int penguinNumber, Direction direction, int nbSteps)
	{
		int[] peng = penguins.get(player);
		int total = 0;
		for(int i = 0; i < penguinNumber; i++) //We add the moves of all preceding penguins
			total += getPenguinTotalMoves(player, i);
		for(Direction d : Direction.values())
		{
			if(d.equals(direction))
				return total+nbSteps-1;
			else
				total += getNbMoves(player, penguinNumber, d);
		}
		return -1;
	}

	public boolean getCan_play_red()
	{
		return can_play_red;
	}

	public boolean getCan_play_blue()
	{
		return can_play_blue;
	}

	public Player getCurrent_player()
	{
		return current_player;
	}

	public int getScore(Player player)
	{
		return score.get(player);
	}

}
